using System;
using EPocalipse.IFilter;
using System.Runtime.InteropServices;

namespace Indexer.Engine.Parsers
{
	public abstract class IFilterParser: ParserBase
	{
		/*
		[DllImport("query.dll", CharSet = CharSet.Unicode)]
		private extern static IFilterReturnCode LoadIFilter(string pwcsPath, [MarshalAs(UnmanagedType.IUnknown)] object pUnkOuter, ref IFilter ppIUnk);

		private static IFilter InternalLoadIFilter(string filename)
		{
			object outer = null;
			IFilter filter = null;

			// Try to load the corresponding IFilter
			IFilterReturnCode resultLoad = LoadIFilter(filename,  outer, ref filter);
			if (resultLoad != IFilterReturnCode.S_OK)
			{
				return null;
			}
			return filter;
		}
		
		private static IFilter LoadAndInitIFilter(string fileName)
		{
			IFilter result = InternalLoadIFilter(fileName);
			if (result != null)
			{
		        IFILTER_FLAGS flags;
		        IFILTER_INIT iflags =
							IFILTER_INIT.CANON_HYPHENS |
							IFILTER_INIT.CANON_PARAGRAPHS |
							IFILTER_INIT.CANON_SPACES |
		        			IFILTER_INIT.APPLY_CRAWL_ATTRIBUTES |
							IFILTER_INIT.APPLY_INDEX_ATTRIBUTES |
		        			IFILTER_INIT.APPLY_OTHER_ATTRIBUTES |
							IFILTER_INIT.HARD_LINE_BREAKS |
		        			IFILTER_INIT.SEARCH_LINKS |
							IFILTER_INIT.FILTER_OWNED_VALUE_OK;
		
		        if (result.Init(iflags, 0, IntPtr.Zero, out flags) != IFilterReturnCode.S_OK)
		        {
			        System.Runtime.InteropServices.Marshal.ReleaseComObject(result);
			        result = null;
		        }
			}
			return result;
		}
		*/
		
		public IFilterParser(string extension): base(extension)
		{
		}
		
		protected override bool GetSupportsCopy()
		{
			return true;
		}
		
		public override void Copy(string fileName, System.IO.TextWriter output)
		{
			IFilter filter = FilterLoader.LoadAndInitIFilter(fileName, Extension);
			if (filter == null)
			{
				throw new ArgumentException(String.Format("No filter available for \"{0}\"", Extension));
			}
			
			Writer writer = new Writer(output);
			try
			{
				try
				{
					STAT_CHUNK chunk = new STAT_CHUNK();
					char[] buffer = new char[64 /* TODO: TUNE: */ * 1024];
					
					while (filter.GetChunk(out chunk) == IFilterReturnCode.S_OK)
					{
						if ((chunk.flags & CHUNKSTATE.CHUNK_TEXT) == CHUNKSTATE.CHUNK_TEXT)
						{
							switch (chunk.breakType)
							{
								case CHUNK_BREAKTYPE.CHUNK_EOW:
									// TODO: this does not work well for PowerPoint documents: 
									// writer.Write(' ');
									// TODO: this breaks some words into pieces
									// writer.WriteLine();
									break;
									
								case CHUNK_BREAKTYPE.CHUNK_EOS:
								case CHUNK_BREAKTYPE.CHUNK_EOP:
								case CHUNK_BREAKTYPE.CHUNK_EOC:
									writer.WriteLine();
									break;
							}
							
							while (true)
							{
								uint bufferSize = (uint)buffer.Length;
								IFilterReturnCode result = filter.GetText(ref bufferSize, buffer);
								if ((result == IFilterReturnCode.S_OK) || (result == IFilterReturnCode.FILTER_S_LAST_TEXT))
								{
									if (bufferSize > buffer.Length)
									{
										// Should never happen, but it happens !
										bufferSize = (uint)buffer.Length;
									}
									writer.Write(buffer, (int)bufferSize);
									
									if (result == IFilterReturnCode.FILTER_S_LAST_TEXT)
									{
										break;
									}
								}
								else
								{
									break;
								}
							}
						}
					}
				}
				finally
				{
			        System.Runtime.InteropServices.Marshal.ReleaseComObject(filter);
			        filter = null;
					System.GC.Collect();
					System.GC.WaitForPendingFinalizers();
				}
			}	
			finally
			{
				writer.Close();
			}
		}
		
		protected void RegisterIFilter(string dllName, Guid filterPersistClass)
		{
			EPocalipse.IFilter.FilterLoader.AddExtensionToCache(Extension, dllName, "{" + filterPersistClass.ToString() + "}");
		}
	}
	
	public class PdfParser: IFilterParser
	{
		public PdfParser(): base(".pdf")
		{
		}
		
		public override bool Accepts(string fileName, byte[] previewBuffer)
		{
			return BufferStartsWith(previewBuffer, "%PDF-1.");
		}
		
		public override bool Accepts(string fileName, string previewBuffer)
		{
			return BufferStartsWith(previewBuffer, "%PDF-1.");
		}
	}
	
	public class OfficeParser: IFilterParser
	{
		private string _sourceExtension;
		
		public OfficeParser(string extension): base(extension)
		{
			_sourceExtension = extension;
		}
		
		public OfficeParser(string sourceExtension, string extension): base(extension)
		{
			_sourceExtension = sourceExtension;
		}
		
		public override bool Accepts(string fileName, byte[] previewBuffer)
		{
			if (!FileHasExtension(fileName, _sourceExtension))
			{
				return false;
			}
			// TODO: recognize also other format containers (not only OLE Structured Storage)
			return BufferStartsWith(previewBuffer, new byte[]{0xD0, 0xCF, 0x11, 0xE0, 0xA1, 0xB1, 0x1A, 0xE1});
		}
		
		public override bool Accepts(string fileName, string previewBuffer)
		{
			return false;
		}
	}
	
	public class HtmlParser: IFilterParser
	{
		public HtmlParser(): base(".htm")
		{
		}
		
		public override bool Accepts(string fileName, byte[] previewBuffer)
		{
			if ((!FileHasExtension(fileName, ".htm")) && (!FileHasExtension(fileName, ".html")))
			{
				return false;
			}
			return BufferContains(previewBuffer, "<html", false);
		}
		
		public override bool Accepts(string fileName, string previewBuffer)
		{
			if ((!FileHasExtension(fileName, ".htm")) && (!FileHasExtension(fileName, ".html")))
			{
				return false;
			}
			return BufferContains(previewBuffer, "<html", false);
		}
	}
}
