using System;
using System.Drawing;
using System.Windows.Forms;

namespace Indexer.Gui
{
	/// <summary>
	/// Description of MessageForm.
	/// </summary>
	public partial class MessageForm : Form
	{
		private IWin32Window _owner;
		
		public MessageForm(IWin32Window owner)
		{
			_owner = owner;
			
			//
			// The InitializeComponent() call is required for Windows Forms designer support.
			//
			InitializeComponent();
			
			//
			// TODO: Add constructor code after the InitializeComponent() call.
			//
			this.iconBox.Image = System.Drawing.IconHelper.GetBitmap(System.Drawing.SystemIcons.Information);
			
			new FocusFix(this);
		}
		
		#region UI construction
		private int _height = 10; // TODO: TUNE: top offset
		private int _width = 0;
		
		private System.Collections.Generic.List<System.Windows.Forms.TextBox> _scrollers = new System.Collections.Generic.List<System.Windows.Forms.TextBox>();
		
		private void AddGroupSeparator()
		{
			_height += 5; 	// TODO: TUNE:
		}
		
		private void AddSingleLineGroup(string caption, string text)
		{
			AddMultiLineGroup(caption, new string[]{text}, 1, false);
		}
		
		private static string Normalized(string text)
		{
			if (String.IsNullOrEmpty(text))
			{
				return "";
			}
			else if (text.StartsWith("\"") && text.EndsWith("\""))
			{
				return text.Substring(1, text.Length - 2);
			}
			else
			{
				return text;
			}
		}
		
		private void MeasureString(System.Windows.Forms.Control control, string text, int leftOffset, int rightOffset)
		{
			System.Drawing.Graphics gc = control.CreateGraphics();
			try
			{
				int w = leftOffset + gc.MeasureString(text, control.Font).ToSize().Width + rightOffset;
				if (w > _width)
				{
					_width = w;
				}
			}
			finally
			{
				gc.Dispose();
			}
		}
		
		private enum LineGroup
		{
			Undefined,
			Text,
			HyperLink
		}
		
		private LineGroup GetLineGroup(string line)
		{
			line = Normalized(line);
			if (line.Contains("http://"))
			{
				return LineGroup.HyperLink;
			}
			else
			{
				return LineGroup.Text;
			}
		}
		
		const int leftOffset = 3; // TODO: TUNE;
			
		private bool AddMultiLineText(string[] text, int startIndex, int count, int visibleCount, LineGroup style, bool scrollable)
		{
			if ((style == LineGroup.Undefined) || (count <= 0))
			{
				return false;
			}
			
			switch (style)
			{
				case LineGroup.Text:
					{
						System.Windows.Forms.TextBox editBox = new System.Windows.Forms.TextBox();
						
						editBox.BorderStyle = BorderStyle.None;
						editBox.WordWrap = false;
						
						System.Text.StringBuilder lines = new System.Text.StringBuilder();
						int toGo = count;
						for(int i = startIndex; toGo > 0; i++, toGo--)
						{
							string line = Normalized(text[i]);
							
							MeasureString(editBox, line, leftOffset, leftOffset);
							
							if (toGo > 1)
							{
								lines.AppendLine(line);
							}
							else
							{
								lines.Append(line);
							}
						}
					
						if (scrollable)
						{
							editBox.ScrollBars = ScrollBars.Vertical;
						}
						
						editBox.Left = leftOffset;
						editBox.Top = _height;
						editBox.ReadOnly = true;
						this.containerPanel.Controls.Add(editBox);
			
						editBox.Text = "a";
						int rowHeight = editBox.Height;
			
						if (count > 1)
						{
							editBox.Multiline = true;
							_scrollers.Add(editBox);
						}
						
						editBox.Text = lines.ToString();
						editBox.ClientSize = new Size(_width - leftOffset - leftOffset, visibleCount * rowHeight);
						
						_height += editBox.Height;
						
					}
					break;
					
				case LineGroup.HyperLink:
					{
						int rowHeight;
						
						// calculate the "normal" row height
						{
							System.Windows.Forms.TextBox editBox = new System.Windows.Forms.TextBox();
							editBox.BorderStyle = BorderStyle.None;
							editBox.WordWrap = false;
							editBox.Left = leftOffset;
							editBox.Top = _height;
							editBox.ReadOnly = true;
							this.containerPanel.Controls.Add(editBox);
							editBox.Text = "a";
							rowHeight = editBox.Height;
							editBox.Dispose();
						}
						
						for(int i = startIndex; count > 0; i++, count--)
						{
							string line = Normalized(text[i]);
							
							System.Windows.Forms.LinkLabel editBox = new System.Windows.Forms.LinkLabel();
							
							MeasureString(editBox, line, leftOffset, leftOffset);
							
							int linkStart = line.IndexOf("http://");
							int linkLength = 0;
							bool searchLinkEnd = true;
							while (searchLinkEnd && (linkStart + linkLength < line.Length))
							{
								switch (line[linkStart + linkLength])
								{
									// TODO: TUNE: link terminators
									case ')':
									case ' ':
										searchLinkEnd = false;
										break;
										
									default:
										linkLength++;
										break;
								}
							}
							
							editBox.Left = leftOffset;
							editBox.Top = _height;
							editBox.Text = line;
							editBox.TextAlign = ContentAlignment.MiddleLeft;
							editBox.LinkArea = new System.Windows.Forms.LinkArea(linkStart, linkLength);
							this.containerPanel.Controls.Add(editBox);
							editBox.ClientSize = new Size(_width /* - leftOffset - leftOffset*/, rowHeight);
							editBox.LinkClicked += this.GenericLinkClicked;
							
							_height += editBox.Height;	// TODO: rowHeight
						}
					}
					break;
					
				default:
					// TODO: implement
					throw new NotImplementedException();
			}
			return true;
		}
		
		private void AddMultiLineText(string[] text, int visibleRowCount, bool scrollable)
		{
			// split text array into sub arrays belonging to the same line group
			LineGroup currentGroup = LineGroup.Undefined;
			int groupStartIndex = 0;
			int index = 0;
			bool singleGroup = true;
			while (index < text.Length)
			{
				LineGroup group = GetLineGroup(text[index]);
				if (group == currentGroup)
				{
					index++;
				}
				else
				{
					if (AddMultiLineText(text, groupStartIndex, index - groupStartIndex, index - groupStartIndex, currentGroup, false))
					{
						singleGroup = false;
					}
					groupStartIndex = index;
					currentGroup = group;
					index++;
				}
			}
			AddMultiLineText(text, groupStartIndex, index - groupStartIndex, singleGroup ? visibleRowCount : index - groupStartIndex, currentGroup, singleGroup ? scrollable : false);
		}
		
		private void AddMultiLineGroup(string caption, string[] text, int visibleRowCount, bool scrollable)
		{
			System.Windows.Forms.Label group = new System.Windows.Forms.Label();
			group.Top = _height;
			this.containerPanel.Controls.Add(group);
			group.Font = new System.Drawing.Font(group.Font, FontStyle.Bold);

			group.AutoSize = false;
			group.Text = caption + ":";
			group.AutoSize = true;

			MeasureString(group, group.Text, 0, 0);
			
			_height += group.Height;
			
			AddMultiLineText(text, visibleRowCount, scrollable);
		}
		
		private void FreezeValues()
		{
			this.Height = this.Height -this.containerPanel.Height + (this.containerPanel.Height - this.containerPanel.ClientSize.Height) + _height;
			this.Width = this.Width - this.containerPanel.Width + (this.containerPanel.Width - this.containerPanel.ClientSize.Width) + _width;

			foreach(System.Windows.Forms.TextBox editBox in _scrollers)
			{
				Size size = editBox.ClientSize;
				size.Width = _width - leftOffset - leftOffset;
				editBox.ClientSize = size;
			}
		}
		#endregion
		
		#region Public interface convertors
		private bool _groupSeparatorNeeded = false;
		
		private const int MaxMultiLineCount = 5;	// TODO: TUNE: for groups with more that MaxMultiLineCount entries, scroll box is used
		
		private void AddGroup(string caption, string[] values, bool singleLine)
		{
			if ((values == null) || (values.Length == 0))
			{
				return;
			}
			
			if (_groupSeparatorNeeded)
			{
				AddGroupSeparator();
			}
			
			if (singleLine && (values.Length == 1))
			{
				AddSingleLineGroup(caption, values[0]);
			}
			else
			{
				if (values.Length > MaxMultiLineCount)
				{
					AddMultiLineGroup(caption, values, MaxMultiLineCount, true);
				}
				else
				{
					AddMultiLineGroup(caption, values, values.Length, false);
				}
			}
			_groupSeparatorNeeded = true;
		}
		
		private string _groupName = null;
		private System.Collections.Generic.List<string> _groupedValues = null;
		private bool _singleLineGroup = false;
		
		private void CloseGroup()
		{
			if (_groupedValues != null)
			{
				AddGroup(_groupName, _groupedValues.ToArray(), _singleLineGroup);
				_groupedValues = null;
			}
		}
		#endregion
		
		#region Public interface
		public void AddGroup(string groupName)
		{
			CloseGroup();
			
			_groupedValues = new System.Collections.Generic.List<string>();
			_groupName = groupName;
			_singleLineGroup = false;
		}
		
		public void AddValue(string text)
		{
			if (!String.IsNullOrEmpty(text))
			{
				_groupedValues.Add(text);
			}
		}
		
		public void Add(string groupName, string text)
		{
			AddGroup(groupName);
			_singleLineGroup = true;
			AddValue(text);
		}
		
		public new bool Show()
		{
			CloseGroup();
			FreezeValues();
			return this.ShowDialog(_owner) == DialogResult.OK;
		}
		#endregion
		
		void ButtonPanelResize(object sender, EventArgs e)
		{
			this.okButton.Left = (this.buttonPanel.ClientSize.Width - this.okButton.Width) / 2;
		}
		
		void GenericLinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
		{
			if (e.Button == MouseButtons.Left)
			{
				string url = (sender as LinkLabel).Text.Substring(e.Link.Start, e.Link.Length);
				e.Link.Visited = true;
				System.Diagnostics.Process.Start(url);
			}
		}
	}
}
