using System;

namespace Structure.Graph2D.IO
{
	public enum EdgeStyle
	{
		Normal,
		Dotted,
		Dashed
	}
	
	public enum ArrowStyle
	{
		None,
		Normal,
		Short
	}
	
	public class Edge
	{
		public int SourceId;
		public int TargetId;
		public bool SmoothBends;
		public EdgeStyle Style = EdgeStyle.Normal;
		public int Width = 1;
		public System.Drawing.Color Color;
		public ArrowStyle TargetArrow;
		public string Text;
		public System.Drawing.Color BackgroundColor;
		public System.Drawing.Color TextColor;
		public System.Drawing.Font Font;
		public bool TextVisible = true;
		public string Class;
		
		public System.Drawing.Point[] Connectors = null;
		
		public string Description;
		public string Url;
	
		public Edge Clone()
		{
			Edge result = new Edge();
			result.SourceId = this.SourceId;
			result.TargetId = this.TargetId;
			result.SmoothBends = this.SmoothBends;
			result.Style = this.Style;
			result.Width = this.Width;
			result.Color = this.Color;
			result.TargetArrow = this.TargetArrow;
			result.Text = this.Text;
			result.BackgroundColor = this.BackgroundColor;
			result.TextColor = this.TextColor;
			result.Font = this.Font;
			result.TextVisible = this.TextVisible;
			result.Class = this.Class;
			result.Description = this.Description;
			result.Url = this.Url;
			result.AddConnectors(this.Connectors);
			return result;
		}

		public void AddConnectors(System.Drawing.Point[] connectors)
		{
			if (connectors != null)
			{
				foreach(System.Drawing.Point connector in connectors)
				{
					AddConnector(connector);
				}
			}
		}
		
		public void AddConnector(System.Drawing.Point connector)
		{
			if (Connectors == null)
			{
				Connectors = new System.Drawing.Point[1];
			}
			else
			{
				System.Array.Resize(ref Connectors, Connectors.Length + 1);
			}
			Connectors[Connectors.Length - 1] = connector;
		}
		
		public Edge()
		{
			Color = System.Drawing.Color.Black;
			TargetArrow = ArrowStyle.Normal;
			BackgroundColor = System.Drawing.Color.Empty;
			TextColor = System.Drawing.Color.Black;
		}
	}
}
