using System;

namespace System.Collections
{
	// TODO: TUNE: use larger type if more bits are needed
	using CoerceType = System.Int32;
	
	public class MultiBitArray
	{
		private int _count;
		private System.Collections.BitArray _bits;
		private int _bitsPerItem;
		
		public MultiBitArray(int count, CoerceType maxValue)
		{
			if (maxValue <= 0)
			{
				throw new ArgumentOutOfRangeException();
			}
			
			CoerceType beyondLimit = 2;
			_bitsPerItem = 1;
			while (maxValue >= beyondLimit)
			{
				_bitsPerItem++;
				beyondLimit = beyondLimit << 1;
			}
			
			_count = count;
			_bits = new System.Collections.BitArray(_count * _bitsPerItem);
		}
		
		public CoerceType this[int index]
		{
			get 
			{
				index *= _bitsPerItem;
				CoerceType result = 0;
				for(int i = _bitsPerItem; i > 0; i--)
				{
					result = (result << 1) | (_bits[index] ? 1 : 0);
					index++;
				}
				return result;
			}
			set 
			{
				index = ((index + 1) * _bitsPerItem) - 1;
				for(int i = _bitsPerItem; i > 0; i--)
				{
					_bits[index] = ((value & 1) != 0);
					value = value >> 1;
					index--;
				}
			}
		}
		
		public int Count 
		{
			get 
			{
				return _count;
			}
		}
	}
}
