using System;
using System.Collections.Generic;
using System.Text;

namespace System.IO.Compression
{
	public class NullFile : ICompressedFile
	{
		private bool pFolderOwned;
		private string pFolder = null;
		public string Folder
		{
			get
			{
				return pFolder;
			}
		}

		private bool pReadOnly = false;
		public bool ReadOnly
		{
			get
			{
				return pReadOnly;
			}
			set
			{
				pReadOnly = value;
			}
		}

		public NullFile()
		{
			Open(null, FileMode.Create, FileAccess.ReadWrite);
		}
		
		public NullFile(string folderName)
		{
			Open(folderName, FileMode.Open, System.IO.FileAccess.Read);
		}

		public void Open(string ArchiveFile, System.IO.FileMode mode, System.IO.FileAccess access)
		{
			if (String.IsNullOrEmpty(ArchiveFile))
			{
				pFolderOwned = true;
				pFolder = System.IO.Path.GetTempFileName();
				System.IO.File.Delete(pFolder);
				System.IO.Directory.CreateDirectory(pFolder);
			}
			else
			{
				pFolderOwned = false;
				pFolder = System.IO.Path.GetFullPath(ArchiveFile);
			}
		}

		public string CompressedNameToTemporaryName(string fileName)
		{
			return System.IO.Path.Combine(pFolder, fileName);
		}
		
		public string TemporaryNameToCompressedName(string fileName)
		{
			return System.IO.PathBuilder.GetRelativePath(pFolder, fileName);
		}
		
		public string CreateFile(string fileName)
		{
			string result = CompressedNameToTemporaryName(fileName);
			System.IO.Directory.CreateDirectory(System.IO.Path.GetDirectoryName(result));
			System.IO.File.Create(result).Dispose();
			return result;
		}
		
		#region Implementation of IDisposable
		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		private void Dispose(bool disposing)
		{
			if (pFolder != null)
			{
				if (pFolderOwned)
				{
					System.IO.Directory.Delete(pFolder, true);
				}
				pFolder = null;
			}
		}

		~NullFile()
		{
			Dispose(false);
		}
		#endregion
	}
}
