using System;

namespace System.Web
{
	public class ArchiveWriter
	{
		private class PageWriter: System.IO.TextWriter
		{
			System.IO.TextWriter _output;
			public PageWriter(System.IO.TextWriter output)
			{
				_output = output;
			}
			
			public override void Close()
			{
				Flush();
			}
			
			protected override void Dispose(bool disposing)
			{
				if (disposing)
				{
					Close();
				}
			}
			
			public override void Flush()
			{
				_output.Flush();
			}
			
			public override void Write(char[] buffer)
			{
				_output.Write(buffer);
			}
			
			public override void WriteLine(char[] buffer, int index, int count)
			{
				_output.WriteLine(buffer, index, count);
			}
			
			public override void WriteLine()
			{
				_output.WriteLine();
			}
			
			public override void WriteLine(string value)
			{
				_output.WriteLine(value);
			}
			
			public override System.Text.Encoding Encoding 
			{
				get 
				{
					return _output.Encoding; 
				}
			}
			
			public override string NewLine 
			{
				get 
				{ 
					return _output.NewLine;
				}
			}
		}
		
		private System.IO.TextWriter _output;
		private bool _closeOutput = false;
		private int _idSeed = 0;
		private string _boundary;
		private string _mainId;
		
		private string GenUid()
		{
			return System.Convert.ToBase64String(Guid.NewGuid().ToByteArray()).Replace("+", "").Replace("=", "").Replace("/", "");
		}
		
		public ArchiveWriter(System.IO.TextWriter output)
		{
			_output = output;
			_boundary = "----------" + GenUid();
			_mainId = String.Format("<archiveWriter.{0}@localhost>", GenUid());
			
			_output.WriteLine(String.Format("Content-Type: multipart/related; start={0}; boundary={1}", _mainId, _boundary));
			// TODO: is it needed?
			// _output.WriteLine("Content-Location: http://dummypage/archive");
			/* TODO:
			 * Subject: =?utf-8?Q?Overview=20(MID=20Profile)?=
			 */
			_output.WriteLine("MIME-Version: 1.0");
		}
		
		public ArchiveWriter(string fileName)
			: this(new System.IO.StreamWriter(fileName, false /* TODO: what about encoding? */))
		{
			_closeOutput = true;
		}
		
		public string CreateID(string extension)
		{
			_idSeed++;
			return String.Format("_item{0}{1}", _idSeed, extension);
		}
		
		public void Close()
		{
			// TODO: isn't one newline enough?
			_output.WriteLine();
			_output.WriteLine();
			_output.WriteLine("--" + _boundary + "--");
			if (_closeOutput)
			{
				_output.Close();
			}
		}
		
		public System.IO.TextWriter CreatePage(string mime, string name, bool mainPage)
		{
			System.IO.TextWriter result = new PageWriter(_output);
			result.WriteLine();
			result.WriteLine("--" + _boundary);
			result.WriteLine(String.Format("Content-Disposition: inline; filename={0}", name));
			result.WriteLine(String.Format("Content-Type: {0}; name={1}", mime, name));
			if (mainPage)
			{
				result.WriteLine(String.Format("Content-Id: {0}", _mainId));
			}
			result.WriteLine(String.Format("Content-Location: file://temp/{0}", name));
			result.WriteLine("Content-Transfer-Encoding: 8bit");
			result.WriteLine();
			
			return result;
		}
		
		public System.IO.TextWriter CreatePage(string name, bool mainPage)
		{
			return CreatePage("text/html", name, mainPage);
		}
		
		public System.IO.TextWriter CreatePage(string name)
		{
			return CreatePage(name, false);
		}
		
		public void AddPage(string mime, string name, bool mainPage, string contents)
		{
			System.IO.TextWriter w = CreatePage(mime, name, mainPage);
			try
			{
				w.WriteLine(contents);
			}
			finally
			{
				w.Dispose();
			}
		}
		
		public void AddPage(string name, bool mainPage, string contents)
		{
			AddPage("text/html", name, mainPage, contents);
		}
		
		public void AddPage(string name, string contents)
		{
			AddPage(name, false, contents);
		}
	}
}
