using System;
using System.Collections;
using System.Windows.Forms;

namespace System.Windows.Forms.Generic
{
	public class ActionManager<MenuAction> where MenuAction: IComparable, IConvertible
	{
		private Hashtable pMenuItems = new Hashtable();
		private Hashtable pActionChecked = new Hashtable();

		private class ActionCode
		{
			public MenuAction Value;
			public bool Valid;

			public ActionCode(MenuAction value)
			{
				Valid = true;
				Value = value;
			}

			public ActionCode()
			{
				Valid = false;
			}
		}

		public ActionManager()
		{
		}

		public void PerformAction(MenuAction action)
		{
			foreach(DictionaryEntry entry in pMenuItems)
			{
				if(action.CompareTo((MenuAction)entry.Value) == 0)
				{
					MenuItem item = (MenuItem)entry.Key;
					item.Checked = GetActionChecked(action);
					item.PerformClick();
					break;
				}
			}
		}

		private ActionCode GetAction(MenuItem item)
		{
			object result = pMenuItems[item];
			return (result == null) ? new ActionCode() : new ActionCode((MenuAction)result);
		}

		public bool HandlesAction(MenuItem item, MenuAction value)
		{
			ActionCode code = GetAction(item);
			return code.Valid ? (value.CompareTo(code.Value) == 0) : false;
		}

		public void SetAction(MenuItem item, MenuAction value)
		{
			pMenuItems[item] = value;
		}

		private void SetAction(MenuItem item, ActionCode value)
		{
			if (!value.Valid)
			{
				pMenuItems.Remove(item);
			}
			else
			{
				SetAction(item, value.Value);
			}
		}

		public void RemoveAction(Menu.MenuItemCollection items)
		{
			foreach (MenuItem item in items)
			{
				RemoveAction(item);
			}
		}

		public void RemoveAction(MenuItem item)
		{
			RemoveAction(item.MenuItems);
			pMenuItems.Remove(item);
		}

		public bool GetActionChecked(MenuAction action)
		{
			object result = pActionChecked[action];
			return (result == null) ? false : (bool)result;
		}

		public void SetActionChecked(MenuAction action, bool value)
		{
			pActionChecked[action] = value;
		}

		private void CopyAction(MenuItem source, MenuItem destination)
		{
			SetAction(destination, GetAction(source));
			for(int i = 0; i < source.MenuItems.Count; i++)
			{
				CopyAction(source.MenuItems[i], destination.MenuItems[i]);
			}
		}

		public MenuItem CloneMenu(MenuItem item)
		{
			MenuItem result = item.CloneMenu();
			CopyAction(item, result);
			return result;
		}

		public void GenericMenuPopup(Menu sender)
		{
			foreach(MenuItem item in sender.MenuItems)
			{
				ActionCode code = GetAction(item);
				if (code.Valid)
				{
					item.Checked = GetActionChecked(code.Value);
					EnableMenuItem(item, code.Value);
				}
				GenericMenuPopup(item);
			}
		}

		public delegate void MenuItemEnabler(MenuItem item, MenuAction action);
		public MenuItemEnabler EnableMenuItem;

		public MenuItem FindActionHandler(Menu sender, MenuAction action)
		{
			foreach (MenuItem item in sender.MenuItems)
			{
				if (HandlesAction(item, action))
				{
					return item;
				}
				MenuItem result = FindActionHandler(item, action);
				if (result != null)
				{
					return result;
				}
			}
			return null;
		}

		private void GenericMenuPopup(object sender, System.EventArgs e)
		{
			GenericMenuPopup((Menu)sender);
		}

	}
}
